"""
Python module generated from Java source file com.google.common.base.Preconditions

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Preconditions:
    """
    Static convenience methods that help a method or constructor check whether it was invoked
    correctly (that is, whether its *preconditions* were met).
    
    If the precondition is not met, the `Preconditions` method throws an unchecked exception
    of a specified type, which helps the method in which the exception was thrown communicate that
    its caller has made a mistake. This allows constructs such as
    
    ````public static double sqrt(double value) {
      if (value < 0) {
        throw new IllegalArgumentException("input is negative: " + value);`
      // calculate square root
    }
    }```
    
    to be replaced with the more compact
    
    ````public static double sqrt(double value) {
      checkArgument(value >= 0, "input is negative: %s", value);
      // calculate square root`
    }```
    
    so that a hypothetical bad caller of this method, such as:
    
    ````void exampleBadCaller() {
      double d = sqrt(-1.0);`
    }```
    
    would be flagged as having called `sqrt()` with an illegal argument.
    
    <h3>Performance</h3>
    
    Avoid passing message arguments that are expensive to compute; your code will always compute
    them, even though they usually won't be needed. If you have such arguments, use the conventional
    if/throw idiom instead.
    
    Depending on your message arguments, memory may be allocated for boxing and varargs array
    creation. However, the methods of this class have a large number of overloads that prevent such
    allocations in many common cases.
    
    The message string is not formatted unless the exception will be thrown, so the cost of the
    string formatting itself should not be a concern.
    
    As with any performance concerns, you should consider profiling your code (in a production
    environment if possible) before spending a lot of effort on tweaking a particular element.
    
    <h3>Other types of preconditions</h3>
    
    Not every type of precondition failure is supported by these methods. Continue to throw
    standard JDK exceptions such as java.util.NoSuchElementException or UnsupportedOperationException in the situations they are intended for.
    
    <h3>Non-preconditions</h3>
    
    It is of course possible to use the methods of this class to check for invalid conditions
    which are *not the caller's fault*. Doing so is **not recommended** because it is
    misleading to future readers of the code and of stack traces. See <a
    href="https://github.com/google/guava/wiki/ConditionalFailuresExplained">Conditional failures
    explained</a> in the Guava User Guide for more advice. Notably, Verify offers assertions
    similar to those in this class for non-precondition checks.
    
    <h3>`java.util.Objects.requireNonNull()`</h3>
    
    Projects which use `com.google.common` should generally avoid the use of java.util.Objects.requireNonNull(Object). Instead, use whichever of .checkNotNull(Object) or Verify.verifyNotNull(Object) is appropriate to the situation.
    (The same goes for the message-accepting overloads.)
    
    <h3>Only `%s` is supported</h3>
    
    `Preconditions` uses Strings.lenientFormat to format error message template
    strings. This only supports the `"%s"` specifier, not the full range of java.util.Formatter specifiers. However, note that if the number of arguments does not match the
    number of occurrences of `"%s"` in the format string, `Preconditions` will still
    behave as expected, and will still include all argument values in the error message; the message
    will simply not be formatted exactly as intended.
    
    <h3>More information</h3>
    
    See the Guava User Guide on <a
    href="https://github.com/google/guava/wiki/PreconditionsExplained">using `Preconditions`</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    @staticmethod
    def checkArgument(expression: bool) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.

        Arguments
        - expression: a boolean expression

        Raises
        - IllegalArgumentException: if `expression` is False
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessage: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.

        Arguments
        - expression: a boolean expression
        - errorMessage: the exception message to use if the check fails; will be converted to a
            string using String.valueOf(Object)

        Raises
        - IllegalArgumentException: if `expression` is False
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, *errorMessageArgs: Tuple["Object", ...]) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.

        Arguments
        - expression: a boolean expression
        - errorMessageTemplate: a template for the exception message should the check fail. The
            message is formed by replacing each `%s` placeholder in the template with an
            argument. These are matched by position - the first `%s` gets `errorMessageArgs[0]`, etc. Unmatched arguments will be appended to the formatted message in
            square braces. Unmatched placeholders will be left as-is.
        - errorMessageArgs: the arguments to be substituted into the message template. Arguments
            are converted to strings using String.valueOf(Object).

        Raises
        - IllegalArgumentException: if `expression` is False
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: str) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: str, p2: str) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: str, p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: str, p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: str, p2: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: str) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: str) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: int, p2: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: "Object", p2: str) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: "Object", p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: "Object", p2: int) -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: "Object", p2: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: "Object", p2: "Object", p3: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkArgument(expression: bool, errorMessageTemplate: str, p1: "Object", p2: "Object", p3: "Object", p4: "Object") -> None:
        """
        Ensures the truth of an expression involving one or more parameters to the calling method.
        
        See .checkArgument(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.

        Arguments
        - expression: a boolean expression

        Raises
        - IllegalStateException: if `expression` is False

        See
        - Verify.verify Verify.verify()
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessage: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.

        Arguments
        - expression: a boolean expression
        - errorMessage: the exception message to use if the check fails; will be converted to a
            string using String.valueOf(Object)

        Raises
        - IllegalStateException: if `expression` is False

        See
        - Verify.verify Verify.verify()
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, *errorMessageArgs: Tuple["Object", ...]) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.

        Arguments
        - expression: a boolean expression
        - errorMessageTemplate: a template for the exception message should the check fail. The
            message is formed by replacing each `%s` placeholder in the template with an
            argument. These are matched by position - the first `%s` gets `errorMessageArgs[0]`, etc. Unmatched arguments will be appended to the formatted message in
            square braces. Unmatched placeholders will be left as-is.
        - errorMessageArgs: the arguments to be substituted into the message template. Arguments
            are converted to strings using String.valueOf(Object).

        Raises
        - IllegalStateException: if `expression` is False

        See
        - Verify.verify Verify.verify()
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: str) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: str, p2: str) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: str, p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: str, p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: str, p2: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: str) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: str) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: int, p2: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: "Object", p2: str) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: "Object", p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: "Object", p2: int) -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: "Object", p2: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: "Object", p2: "Object", p3: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkState(expression: bool, errorMessageTemplate: str, p1: "Object", p2: "Object", p3: "Object", p4: "Object") -> None:
        """
        Ensures the truth of an expression involving the state of the calling instance, but not
        involving any parameters to the calling method.
        
        See .checkState(boolean, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.

        Arguments
        - reference: an object reference

        Returns
        - the non-null reference that was validated

        Raises
        - NullPointerException: if `reference` is null

        See
        - Verify.verifyNotNull Verify.verifyNotNull()
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessage: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.

        Arguments
        - reference: an object reference
        - errorMessage: the exception message to use if the check fails; will be converted to a
            string using String.valueOf(Object)

        Returns
        - the non-null reference that was validated

        Raises
        - NullPointerException: if `reference` is null

        See
        - Verify.verifyNotNull Verify.verifyNotNull()
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, *errorMessageArgs: Tuple["Object", ...]) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.

        Arguments
        - reference: an object reference
        - errorMessageTemplate: a template for the exception message should the check fail. The
            message is formed by replacing each `%s` placeholder in the template with an
            argument. These are matched by position - the first `%s` gets `errorMessageArgs[0]`, etc. Unmatched arguments will be appended to the formatted message in
            square braces. Unmatched placeholders will be left as-is.
        - errorMessageArgs: the arguments to be substituted into the message template. Arguments
            are converted to strings using String.valueOf(Object).

        Returns
        - the non-null reference that was validated

        Raises
        - NullPointerException: if `reference` is null

        See
        - Verify.verifyNotNull Verify.verifyNotNull()
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: str) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: str, p2: str) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: str, p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: str, p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: str, p2: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: str) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: str) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: int, p2: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: "Object", p2: str) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: "Object", p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: "Object", p2: int) -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: "Object", p2: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: "Object", p2: "Object", p3: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkNotNull(reference: "T", errorMessageTemplate: str, p1: "Object", p2: "Object", p3: "Object", p4: "Object") -> "T":
        """
        Ensures that an object reference passed as a parameter to the calling method is not null.
        
        See .checkNotNull(Object, String, Object...) for details.

        Since
        - 20.0 (varargs overload since 2.0)
        """
        ...


    @staticmethod
    def checkElementIndex(index: int, size: int) -> int:
        """
        Ensures that `index` specifies a valid *element* in an array, list or string of size
        `size`. An element index may range from zero, inclusive, to `size`, exclusive.

        Arguments
        - index: a user-supplied index identifying an element of an array, list or string
        - size: the size of that array, list or string

        Returns
        - the value of `index`

        Raises
        - IndexOutOfBoundsException: if `index` is negative or is not less than `size`
        - IllegalArgumentException: if `size` is negative
        """
        ...


    @staticmethod
    def checkElementIndex(index: int, size: int, desc: str) -> int:
        """
        Ensures that `index` specifies a valid *element* in an array, list or string of size
        `size`. An element index may range from zero, inclusive, to `size`, exclusive.

        Arguments
        - index: a user-supplied index identifying an element of an array, list or string
        - size: the size of that array, list or string
        - desc: the text to use to describe this index in an error message

        Returns
        - the value of `index`

        Raises
        - IndexOutOfBoundsException: if `index` is negative or is not less than `size`
        - IllegalArgumentException: if `size` is negative
        """
        ...


    @staticmethod
    def checkPositionIndex(index: int, size: int) -> int:
        """
        Ensures that `index` specifies a valid *position* in an array, list or string of
        size `size`. A position index may range from zero to `size`, inclusive.

        Arguments
        - index: a user-supplied index identifying a position in an array, list or string
        - size: the size of that array, list or string

        Returns
        - the value of `index`

        Raises
        - IndexOutOfBoundsException: if `index` is negative or is greater than `size`
        - IllegalArgumentException: if `size` is negative
        """
        ...


    @staticmethod
    def checkPositionIndex(index: int, size: int, desc: str) -> int:
        """
        Ensures that `index` specifies a valid *position* in an array, list or string of
        size `size`. A position index may range from zero to `size`, inclusive.

        Arguments
        - index: a user-supplied index identifying a position in an array, list or string
        - size: the size of that array, list or string
        - desc: the text to use to describe this index in an error message

        Returns
        - the value of `index`

        Raises
        - IndexOutOfBoundsException: if `index` is negative or is greater than `size`
        - IllegalArgumentException: if `size` is negative
        """
        ...


    @staticmethod
    def checkPositionIndexes(start: int, end: int, size: int) -> None:
        """
        Ensures that `start` and `end` specify valid *positions* in an array, list or
        string of size `size`, and are in order. A position index may range from zero to `size`, inclusive.

        Arguments
        - start: a user-supplied index identifying a starting position in an array, list or string
        - end: a user-supplied index identifying an ending position in an array, list or string
        - size: the size of that array, list or string

        Raises
        - IndexOutOfBoundsException: if either index is negative or is greater than `size`,
            or if `end` is less than `start`
        - IllegalArgumentException: if `size` is negative
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Skeleton

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Skeleton(AbstractSkeleton):
    """
    Represents a Skeleton.
    
    This interface only represents the normal skeleton type on the server.
    Other skeleton-like entities, such as the WitherSkeleton or the
    Stray are not related to this type.
    """

    def isConverting(self) -> bool:
        """
        Computes whether or not this skeleton is currently in the process of
        converting to a Stray due to it being frozen by powdered snow.

        Returns
        - whether or not the skeleton is converting to a stray.
        """
        ...


    def getConversionTime(self) -> int:
        """
        Gets the amount of ticks until this entity will be converted to a stray
        as a result of being frozen by a powdered snow block.
        
        When this reaches 0, the entity will be converted.

        Returns
        - the conversion time left represented in ticks.

        Raises
        - IllegalStateException: if .isConverting() is False.
        """
        ...


    def setConversionTime(self, time: int) -> None:
        """
        Sets the amount of ticks until this entity will be converted to a stray
        as a result of being frozen by a powdered snow block.
        
        When this reaches 0, the entity will be converted. A value of less than 0
        will stop the current conversion process without converting the current
        entity.

        Arguments
        - time: the new conversion time left before the conversion in ticks.
        """
        ...


    class SkeletonType(Enum):
        """
        A legacy enum that defines the different variances of skeleton-like
        entities on the server.

        Deprecated
        - classes are different types. This interface only remains in
            the Skeleton interface to preserve backwards compatibility.
        """

        NORMAL = 0
        """
        Standard skeleton type.
        """
        WITHER = 1
        """
        Wither skeleton. Generally found in Nether fortresses.
        """
        STRAY = 2
        """
        Stray skeleton. Generally found in ice biomes. Shoots tipped arrows.
        """
        BOGGED = 3
        """
        Bogged skeleton.
        """



"""
Python module generated from Java source file org.bukkit.FireworkEffect

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableList
from com.google.common.collect import ImmutableMap
from enum import Enum
from org.bukkit import *
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.configuration.serialization import SerializableAs
from typing import Any, Callable, Iterable, Tuple


class FireworkEffect(ConfigurationSerializable):
    """
    Represents a single firework effect.
    """

    @staticmethod
    def builder() -> "Builder":
        """
        Construct a firework effect.

        Returns
        - A utility object for building a firework effect
        """
        ...


    def hasFlicker(self) -> bool:
        """
        Get whether the firework effect flickers.

        Returns
        - True if it flickers, False if not
        """
        ...


    def hasTrail(self) -> bool:
        """
        Get whether the firework effect has a trail.

        Returns
        - True if it has a trail, False if not
        """
        ...


    def getColors(self) -> list["Color"]:
        """
        Get the primary colors of the firework effect.

        Returns
        - An immutable list of the primary colors
        """
        ...


    def getFadeColors(self) -> list["Color"]:
        """
        Get the fade colors of the firework effect.

        Returns
        - An immutable list of the fade colors
        """
        ...


    def getType(self) -> "Type":
        """
        Get the type of the firework effect.

        Returns
        - The effect type
        """
        ...


    @staticmethod
    def deserialize(map: dict[str, "Object"]) -> "ConfigurationSerializable":
        """
        Arguments
        - map: the map to deserialize

        Returns
        - the resulting serializable

        See
        - ConfigurationSerializable
        """
        ...


    def serialize(self) -> dict[str, "Object"]:
        ...


    def toString(self) -> str:
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    class Builder:
        """
        This is a builder for FireworkEffects.

        See
        - FireworkEffect.builder()
        """

        def with(self, type: "Type") -> "Builder":
            """
            Specify the type of the firework effect.

            Arguments
            - type: The effect type

            Returns
            - This object, for chaining

            Raises
            - IllegalArgumentException: If type is null
            """
            ...


        def withFlicker(self) -> "Builder":
            """
            Add a flicker to the firework effect.

            Returns
            - This object, for chaining
            """
            ...


        def flicker(self, flicker: bool) -> "Builder":
            """
            Set whether the firework effect should flicker.

            Arguments
            - flicker: True if it should flicker, False if not

            Returns
            - This object, for chaining
            """
            ...


        def withTrail(self) -> "Builder":
            """
            Add a trail to the firework effect.

            Returns
            - This object, for chaining
            """
            ...


        def trail(self, trail: bool) -> "Builder":
            """
            Set whether the firework effect should have a trail.

            Arguments
            - trail: True if it should have a trail, False for no trail

            Returns
            - This object, for chaining
            """
            ...


        def withColor(self, color: "Color") -> "Builder":
            """
            Add a primary color to the firework effect.

            Arguments
            - color: The color to add

            Returns
            - This object, for chaining

            Raises
            - IllegalArgumentException: If color is null
            """
            ...


        def withColor(self, *colors: Tuple["Color", ...]) -> "Builder":
            """
            Add several primary colors to the firework effect.

            Arguments
            - colors: The colors to add

            Returns
            - This object, for chaining

            Raises
            - IllegalArgumentException: If colors is null
            - IllegalArgumentException: If any color is null (may be
                thrown after changes have occurred)
            """
            ...


        def withColor(self, colors: Iterable[Any]) -> "Builder":
            """
            Add several primary colors to the firework effect.

            Arguments
            - colors: An iterable object whose iterator yields the desired
                colors

            Returns
            - This object, for chaining

            Raises
            - IllegalArgumentException: If colors is null
            - IllegalArgumentException: If any color is null (may be
                thrown after changes have occurred)
            """
            ...


        def withFade(self, color: "Color") -> "Builder":
            """
            Add a fade color to the firework effect.

            Arguments
            - color: The color to add

            Returns
            - This object, for chaining

            Raises
            - IllegalArgumentException: If colors is null
            - IllegalArgumentException: If any color is null (may be
                thrown after changes have occurred)
            """
            ...


        def withFade(self, *colors: Tuple["Color", ...]) -> "Builder":
            """
            Add several fade colors to the firework effect.

            Arguments
            - colors: The colors to add

            Returns
            - This object, for chaining

            Raises
            - IllegalArgumentException: If colors is null
            - IllegalArgumentException: If any color is null (may be
                thrown after changes have occurred)
            """
            ...


        def withFade(self, colors: Iterable[Any]) -> "Builder":
            """
            Add several fade colors to the firework effect.

            Arguments
            - colors: An iterable object whose iterator yields the desired
                colors

            Returns
            - This object, for chaining

            Raises
            - IllegalArgumentException: If colors is null
            - IllegalArgumentException: If any color is null (may be
                thrown after changes have occurred)
            """
            ...


        def build(self) -> "FireworkEffect":
            """
            Create a FireworkEffect from the current contents of this
            builder.
            
            To successfully build, you must have specified at least one color.

            Returns
            - The representative firework effect
            """
            ...


    class Type(Enum):
        """
        The type or shape of the effect.
        """

        BALL = 0
        """
        A small ball effect.
        """
        BALL_LARGE = 1
        """
        A large ball effect.
        """
        STAR = 2
        """
        A star-shaped effect.
        """
        BURST = 3
        """
        A burst effect.
        """
        CREEPER = 4
        """
        A creeper-face effect.
        """



"""
Python module generated from Java source file org.bukkit.material.SpawnEgg

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.entity import EntityType
from org.bukkit.inventory.meta import SpawnEggMeta
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class SpawnEgg(MaterialData):
    """
    Represents a spawn egg that can be used to spawn mobs

    Deprecated
    - use SpawnEggMeta
    """

    def __init__(self):
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def __init__(self, data: int):
        """
        Arguments
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def __init__(self, type: "EntityType"):
        ...


    def getSpawnedType(self) -> "EntityType":
        """
        Get the type of entity this egg will spawn.

        Returns
        - The entity type.

        Deprecated
        - This is now stored in SpawnEggMeta.
        """
        ...


    def setSpawnedType(self, type: "EntityType") -> None:
        """
        Set the type of entity this egg will spawn.

        Arguments
        - type: The entity type.

        Deprecated
        - This is now stored in SpawnEggMeta.
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "SpawnEgg":
        ...



"""
Python module generated from Java source file org.bukkit.event.raid.RaidFinishEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Collections
from org.bukkit import Raid
from org.bukkit import World
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.raid import *
from typing import Any, Callable, Iterable, Tuple


class RaidFinishEvent(RaidEvent):
    """
    This event is called when a Raid was complete with a clear result.
    """

    def __init__(self, raid: "Raid", world: "World", winners: list["Player"]):
        ...


    def getWinners(self) -> list["Player"]:
        """
        Returns an immutable list contains all winners.
        
        **Note: Players who are considered as heroes but were not online at the
        end would not be included in this list.**

        Returns
        - winners
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.PortalType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class PortalType(Enum):
    """
    Represents various types of portals that can be made in a world.
    """

    NETHER = 0
    """
    This is a Nether portal, made of obsidian.
    """
    ENDER = 1
    """
    This is an Ender portal.
    """
    CUSTOM = 2
    """
    This is a custom Plugin portal.
    """



"""
Python module generated from Java source file com.google.common.reflect.TypeToken

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Joiner
from com.google.common.base import Predicate
from com.google.common.collect import FluentIterable
from com.google.common.collect import ForwardingSet
from com.google.common.collect import ImmutableList
from com.google.common.collect import ImmutableMap
from com.google.common.collect import ImmutableSet
from com.google.common.collect import Maps
from com.google.common.collect import Ordering
from com.google.common.primitives import Primitives
from com.google.common.reflect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import LazyInit
from java.io import Serializable
from java.lang.reflect import Constructor
from java.lang.reflect import GenericArrayType
from java.lang.reflect import Method
from java.lang.reflect import Modifier
from java.lang.reflect import ParameterizedType
from java.lang.reflect import Type
from java.lang.reflect import TypeVariable
from java.lang.reflect import WildcardType
from java.util import Arrays
from java.util import Comparator
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class TypeToken(TypeCapture, Serializable):
    """
    A Type with generics.
    
    Operations that are otherwise only available in Class are implemented to support
    `Type`, for example .isSubtypeOf, .isArray and .getComponentType.
    It also provides additional utilities such as .getTypes, .resolveType, etc.
    
    There are three ways to get a `TypeToken` instance:
    
    
      - Wrap a `Type` obtained via reflection. For example: `TypeToken.of(method.getGenericReturnType())`.
      - Capture a generic type with a (usually anonymous) subclass. For example:
          ````new TypeToken<List<String>>() {`
    }```
          Note that it's critical that the actual type argument is carried by a subclass. The
          following code is wrong because it only captures the `<T>` type variable of the
          `listType()` method signature; while `<String>` is lost in erasure:
          ````class Util {
      static <T> TypeToken<List<T>> listType() {
        return new TypeToken<List<T>>() {`;
      }
    }
    
    TypeToken<List<String>> stringListType = Util.<String>listType();
    }```
      - Capture a generic type with a (usually anonymous) subclass and resolve it against a context
          class that knows what the type parameters are. For example:
          ````abstract class IKnowMyType<T> {
      TypeToken<T> type = new TypeToken<T>(getClass()) {`;
    }
    new IKnowMyType<String>() {}.type => String
    }```
    
    
    `TypeToken` is serializable when no type variable is contained in the type.
    
    Note to Guice users: `TypeToken` is similar to Guice's `TypeLiteral` class except
    that it is serializable and offers numerous additional utility methods.

    Author(s)
    - Ben Yu

    Since
    - 12.0
    """

    @staticmethod
    def of(type: type["T"]) -> "TypeToken"["T"]:
        """
        Returns an instance of type token that wraps `type`.
        """
        ...


    @staticmethod
    def of(type: "Type") -> "TypeToken"[Any]:
        """
        Returns an instance of type token that wraps `type`.
        """
        ...


    def getRawType(self) -> type["T"]:
        """
        Returns the raw type of `T`. Formally speaking, if `T` is returned by java.lang.reflect.Method.getGenericReturnType, the raw type is what's returned by java.lang.reflect.Method.getReturnType of the same method object. Specifically:
        
        
          - If `T` is a `Class` itself, `T` itself is returned.
          - If `T` is a ParameterizedType, the raw type of the parameterized type is
              returned.
          - If `T` is a GenericArrayType, the returned type is the corresponding array
              class. For example: `List<Integer>[] => List[]`.
          - If `T` is a type variable or a wildcard type, the raw type of the first upper bound
              is returned. For example: `<X extends Foo> => Foo`.
        """
        ...


    def getType(self) -> "Type":
        """
        Returns the represented type.
        """
        ...


    def where(self, typeParam: "TypeParameter"["X"], typeArg: "TypeToken"["X"]) -> "TypeToken"["T"]:
        ...


    def where(self, typeParam: "TypeParameter"["X"], typeArg: type["X"]) -> "TypeToken"["T"]:
        ...


    def resolveType(self, type: "Type") -> "TypeToken"[Any]:
        """
        Resolves the given `type` against the type context represented by this type. For example:
        
        ````new TypeToken<List<String>>() {`.resolveType(
            List.class.getMethod("get", int.class).getGenericReturnType())
        => String.class
        }```
        """
        ...


    def getTypes(self) -> "TypeSet":
        """
        Returns the set of interfaces and classes that this type is or is a subtype of. The returned
        types are parameterized with proper type arguments.
        
        Subtypes are always listed before supertypes. But the reverse is not True. A type isn't
        necessarily a subtype of all the types following. Order between types without subtype
        relationship is arbitrary and not guaranteed.
        
        If this type is a type variable or wildcard, upper bounds that are themselves type variables
        aren't included (their super interfaces and superclasses are).
        """
        ...


    def getSupertype(self, superclass: type["T"]) -> "TypeToken"["T"]:
        """
        Returns the generic form of `superclass`. For example, if this is `ArrayList<String>`, `Iterable<String>` is returned given the input `Iterable.class`.
        """
        ...


    def getSubtype(self, subclass: type[Any]) -> "TypeToken"["T"]:
        """
        Returns subtype of `this` with `subclass` as the raw class. For example, if this is
        `Iterable<String>` and `subclass` is `List`, `List<String>` is
        returned.
        """
        ...


    def isSupertypeOf(self, type: "TypeToken"[Any]) -> bool:
        """
        Returns True if this type is a supertype of the given `type`. "Supertype" is defined
        according to <a
        href="http://docs.oracle.com/javase/specs/jls/se8/html/jls-4.html#jls-4.5.1">the rules for type
        arguments</a> introduced with Java generics.

        Since
        - 19.0
        """
        ...


    def isSupertypeOf(self, type: "Type") -> bool:
        """
        Returns True if this type is a supertype of the given `type`. "Supertype" is defined
        according to <a
        href="http://docs.oracle.com/javase/specs/jls/se8/html/jls-4.html#jls-4.5.1">the rules for type
        arguments</a> introduced with Java generics.

        Since
        - 19.0
        """
        ...


    def isSubtypeOf(self, type: "TypeToken"[Any]) -> bool:
        """
        Returns True if this type is a subtype of the given `type`. "Subtype" is defined
        according to <a
        href="http://docs.oracle.com/javase/specs/jls/se8/html/jls-4.html#jls-4.5.1">the rules for type
        arguments</a> introduced with Java generics.

        Since
        - 19.0
        """
        ...


    def isSubtypeOf(self, supertype: "Type") -> bool:
        """
        Returns True if this type is a subtype of the given `type`. "Subtype" is defined
        according to <a
        href="http://docs.oracle.com/javase/specs/jls/se8/html/jls-4.html#jls-4.5.1">the rules for type
        arguments</a> introduced with Java generics.

        Since
        - 19.0
        """
        ...


    def isArray(self) -> bool:
        """
        Returns True if this type is known to be an array type, such as `int[]`, `T[]`,
        `<? extends Map<String, Integer>[]>` etc.
        """
        ...


    def isPrimitive(self) -> bool:
        """
        Returns True if this type is one of the nine primitive types (including `void`).

        Since
        - 15.0
        """
        ...


    def wrap(self) -> "TypeToken"["T"]:
        """
        Returns the corresponding wrapper type if this is a primitive type; otherwise returns `this` itself. Idempotent.

        Since
        - 15.0
        """
        ...


    def unwrap(self) -> "TypeToken"["T"]:
        """
        Returns the corresponding primitive type if this is a wrapper type; otherwise returns `this` itself. Idempotent.

        Since
        - 15.0
        """
        ...


    def getComponentType(self) -> "TypeToken"[Any]:
        """
        Returns the array component type if this type represents an array (`int[]`, `T[]`,
        `<? extends Map<String, Integer>[]>` etc.), or else `null` is returned.
        """
        ...


    def method(self, method: "Method") -> "Invokable"["T", "Object"]:
        """
        Returns the Invokable for `method`, which must be a member of `T`.

        Since
        - 14.0
        """
        ...


    def constructor(self, constructor: "Constructor"[Any]) -> "Invokable"["T", "T"]:
        """
        Returns the Invokable for `constructor`, which must be a member of `T`.

        Since
        - 14.0
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Returns True if `o` is another `TypeToken` that represents the same Type.
        """
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...


    class TypeSet(ForwardingSet, Serializable):
        """
        The set of interfaces and classes that `T` is or is a subtype of. Object is not
        included in the set if this type is an interface.

        Since
        - 13.0
        """

        def interfaces(self) -> "TypeSet":
            """
            Returns the types that are interfaces implemented by this type.
            """
            ...


        def classes(self) -> "TypeSet":
            """
            Returns the types that are classes.
            """
            ...


        def rawTypes(self) -> set[type["T"]]:
            """
            Returns the raw types of the types in this set, in the same order.
            """
            ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityTransformEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.util import Collections
from org.bukkit.entity import Entity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityTransformEvent(EntityEvent, Cancellable):
    """
    Called when an entity is about to be replaced by another entity.
    """

    def __init__(self, original: "Entity", convertedList: list["Entity"], transformReason: "TransformReason"):
        ...


    def getTransformedEntity(self) -> "Entity":
        """
        Gets the entity that the original entity was transformed to.
        
        This returns the first entity in the transformed entity list.

        Returns
        - The transformed entity.

        See
        - .getTransformedEntities()
        """
        ...


    def getTransformedEntities(self) -> list["Entity"]:
        """
        Gets the entities that the original entity was transformed to.

        Returns
        - The transformed entities.
        """
        ...


    def getTransformReason(self) -> "TransformReason":
        """
        Gets the reason for the conversion that has occurred.

        Returns
        - The reason for conversion that has occurred.
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class TransformReason(Enum):

        CURED = 0
        """
        When a zombie gets cured and a villager is spawned.
        """
        FROZEN = 1
        """
        When an entity is shaking in Powder Snow and a new entity spawns.
        """
        INFECTION = 2
        """
        When a villager gets infected and a zombie villager spawns.
        """
        DROWNED = 3
        """
        When an entity drowns in water and a new entity spawns.
        """
        SHEARED = 4
        """
        When a mooshroom (or MUSHROOM_COW) is sheared and a cow spawns.
        """
        LIGHTNING = 5
        """
        When lightning strikes a entity.
        """
        SPLIT = 6
        """
        When a slime splits into multiple smaller slimes.
        """
        PIGLIN_ZOMBIFIED = 7
        """
        When a piglin (or hoglin) converts to a zombified version from overworld presence.
        """
        METAMORPHOSIS = 8
        """
        When a tadpole converts to a frog
        """
        UNKNOWN = 9
        """
        When reason is unknown.
        """



"""
Python module generated from Java source file org.bukkit.ban.ProfileBanList

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Date
from org.bukkit import BanEntry
from org.bukkit import BanList
from org.bukkit.ban import *
from org.bukkit.profile import PlayerProfile
from typing import Any, Callable, Iterable, Tuple


class ProfileBanList(BanList):
    """
    A BanList targeting player profile bans.
    """

    def addBan(self, target: "PlayerProfile", reason: str, expires: "Date", source: str) -> "BanEntry"["PlayerProfile"]:
        """
        Arguments
        - target: the target of the ban
        - reason: reason for the ban, null indicates implementation default
        - expires: date for the ban's expiration (unban), or null to imply
            forever
        - source: source of the ban, null indicates implementation default

        Returns
        - the entry for the newly created ban, or the entry for the
            (updated) previous ban

        Raises
        - IllegalArgumentException: if ProfilePlayer has an invalid UUID
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerItemConsumeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class PlayerItemConsumeEvent(PlayerEvent, Cancellable):
    """
    This event will fire when a player is finishing consuming an item (food,
    potion, milk bucket).
    
    If the ItemStack is modified the server will use the effects of the new
    item and not remove the original one from the player's inventory.
    
    If the event is cancelled the effect will not be applied and the item will
    not be removed from the player's inventory.
    """

    def __init__(self, player: "Player", item: "ItemStack", hand: "EquipmentSlot"):
        """
        Arguments
        - player: the player consuming
        - item: the ItemStack being consumed
        - hand: the hand that was used
        """
        ...


    def __init__(self, player: "Player", item: "ItemStack"):
        """
        Arguments
        - player: the player consuming
        - item: the ItemStack being consumed

        Deprecated
        - use .PlayerItemConsumeEvent(Player, ItemStack, EquipmentSlot)
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Gets the item that is being consumed. Modifying the returned item will
        have no effect, you must use .setItem(org.bukkit.inventory.ItemStack) instead.

        Returns
        - an ItemStack for the item being consumed
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Set the item being consumed

        Arguments
        - item: the item being consumed
        """
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Get the hand used to consume the item.

        Returns
        - the hand
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.ClickType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.event.inventory import *
from typing import Any, Callable, Iterable, Tuple


class ClickType(Enum):
    """
    What the client did to trigger this action (not the result).
    """

    LEFT = 0
    """
    The left (or primary) mouse button.
    """
    SHIFT_LEFT = 1
    """
    Holding shift while pressing the left mouse button.
    """
    RIGHT = 2
    """
    The right mouse button.
    """
    SHIFT_RIGHT = 3
    """
    Holding shift while pressing the right mouse button.
    """
    WINDOW_BORDER_LEFT = 4
    """
    Clicking the left mouse button on the grey area around the inventory.
    """
    WINDOW_BORDER_RIGHT = 5
    """
    Clicking the right mouse button on the grey area around the inventory.
    """
    MIDDLE = 6
    """
    The middle mouse button, or a "scrollwheel click".
    """
    NUMBER_KEY = 7
    """
    One of the number keys 1-9, correspond to slots on the hotbar.
    """
    DOUBLE_CLICK = 8
    """
    Pressing the left mouse button twice in quick succession.
    """
    DROP = 9
    """
    The "Drop" key (defaults to Q).
    """
    CONTROL_DROP = 10
    """
    Holding Ctrl while pressing the "Drop" key (defaults to Q).
    """
    CREATIVE = 11
    """
    Any action done with the Creative inventory open.
    """
    SWAP_OFFHAND = 12
    """
    The "swap item with offhand" key (defaults to F).
    """
    UNKNOWN = 13
    """
    A type of inventory manipulation not yet recognized by Bukkit.
    
    This is only for transitional purposes on a new Minecraft update, and
    should never be relied upon.
    
    Any ClickType.UNKNOWN is called on a best-effort basis.
    """


    def isKeyboardClick(self) -> bool:
        """
        Gets whether this ClickType represents the pressing of a key on a
        keyboard.

        Returns
        - True if this ClickType represents the pressing of a key
        """
        ...


    def isMouseClick(self) -> bool:
        """
        Gets whether this ClickType represents the pressing of a mouse button

        Returns
        - True if this ClickType represents the pressing of a mouse button
        """
        ...


    def isCreativeAction(self) -> bool:
        """
        Gets whether this ClickType represents an action that can only be
        performed by a Player in creative mode.

        Returns
        - True if this action requires Creative mode
        """
        ...


    def isRightClick(self) -> bool:
        """
        Gets whether this ClickType represents a right click.

        Returns
        - True if this ClickType represents a right click
        """
        ...


    def isLeftClick(self) -> bool:
        """
        Gets whether this ClickType represents a left click.

        Returns
        - True if this ClickType represents a left click
        """
        ...


    def isShiftClick(self) -> bool:
        """
        Gets whether this ClickType indicates that the shift key was pressed
        down when the click was made.

        Returns
        - True if the action uses Shift.
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.ChecksumHashFunction

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.hash import *
from com.google.errorprone.annotations import Immutable
from com.google.j2objc.annotations import J2ObjCIncompatible
from java.io import Serializable
from java.lang.invoke import MethodHandle
from java.lang.invoke import MethodHandles
from java.lang.invoke import MethodType
from java.util.zip import Checksum
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ChecksumHashFunction(AbstractHashFunction, Serializable):
    """
    HashFunction adapter for Checksum instances.

    Author(s)
    - Colin Decker
    """

    def bits(self) -> int:
        ...


    def newHasher(self) -> "Hasher":
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.entity.Arrow

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Color
from org.bukkit.entity import *
from org.bukkit.potion import PotionData
from org.bukkit.potion import PotionEffect
from org.bukkit.potion import PotionEffectType
from org.bukkit.potion import PotionType
from typing import Any, Callable, Iterable, Tuple


class Arrow(AbstractArrow):

    def setBasePotionData(self, data: "PotionData") -> None:
        """
        Sets the underlying potion data

        Arguments
        - data: PotionData to set the base potion state to

        Deprecated
        - Upgraded / extended potions are now their own PotionType use .setBasePotionType instead.
        """
        ...


    def getBasePotionData(self) -> "PotionData":
        """
        Returns the potion data about the base potion

        Returns
        - a PotionData object

        Deprecated
        - Upgraded / extended potions are now their own PotionType use .getBasePotionType() instead.
        """
        ...


    def setBasePotionType(self, type: "PotionType") -> None:
        """
        Sets the underlying potion type

        Arguments
        - type: PotionType to set the base potion state to
        """
        ...


    def getBasePotionType(self) -> "PotionType":
        """
        Returns the potion type about the base potion

        Returns
        - a PotionType object
        """
        ...


    def getColor(self) -> "Color":
        """
        Gets the color of this arrow.

        Returns
        - arrow Color or null if not color is set
        """
        ...


    def setColor(self, color: "Color") -> None:
        """
        Sets the color of this arrow. Will be applied as a tint to its particles.

        Arguments
        - color: arrow color, null to clear the color
        """
        ...


    def hasCustomEffects(self) -> bool:
        """
        Checks for the presence of custom potion effects.

        Returns
        - True if custom potion effects are applied
        """
        ...


    def getCustomEffects(self) -> list["PotionEffect"]:
        """
        Gets an immutable list containing all custom potion effects applied to
        this arrow.
        
        Plugins should check that hasCustomEffects() returns True before calling
        this method.

        Returns
        - the immutable list of custom potion effects
        """
        ...


    def addCustomEffect(self, effect: "PotionEffect", overwrite: bool) -> bool:
        """
        Adds a custom potion effect to this arrow.

        Arguments
        - effect: the potion effect to add
        - overwrite: True if any existing effect of the same type should be
        overwritten

        Returns
        - True if the effect was added as a result of this call
        """
        ...


    def removeCustomEffect(self, type: "PotionEffectType") -> bool:
        """
        Removes a custom potion effect from this arrow.

        Arguments
        - type: the potion effect type to remove

        Returns
        - True if the an effect was removed as a result of this call

        Raises
        - IllegalArgumentException: if this operation would leave the Arrow
        in a state with no Custom Effects and PotionType.UNCRAFTABLE
        """
        ...


    def hasCustomEffect(self, type: "PotionEffectType") -> bool:
        """
        Checks for a specific custom potion effect type on this arrow.

        Arguments
        - type: the potion effect type to check for

        Returns
        - True if the potion has this effect
        """
        ...


    def clearCustomEffects(self) -> None:
        """
        Removes all custom potion effects from this arrow.

        Raises
        - IllegalArgumentException: if this operation would leave the Arrow
        in a state with no Custom Effects and PotionType.UNCRAFTABLE
        """
        ...



from .InventoryType import InventoryType
from .FurnaceStartSmeltEvent import FurnaceStartSmeltEvent
from .BrewEvent import BrewEvent
from .FurnaceBurnEvent import FurnaceBurnEvent
from .PrepareAnvilEvent import PrepareAnvilEvent
from .InventoryInteractEvent import InventoryInteractEvent
from .TradeSelectEvent import TradeSelectEvent
from .DragType import DragType
from .FurnaceSmeltEvent import FurnaceSmeltEvent
from .InventoryAction import InventoryAction
from .InventoryDragEvent import InventoryDragEvent
from .PrepareGrindstoneEvent import PrepareGrindstoneEvent
from .InventoryEvent import InventoryEvent
from .BrewingStandFuelEvent import BrewingStandFuelEvent
from .FurnaceExtractEvent import FurnaceExtractEvent
from .InventoryClickEvent import InventoryClickEvent
from .ClickType import ClickType
from .InventoryCreativeEvent import InventoryCreativeEvent
from .PrepareSmithingEvent import PrepareSmithingEvent
from .CraftItemEvent import CraftItemEvent
from .HopperInventorySearchEvent import HopperInventorySearchEvent
from .PrepareItemCraftEvent import PrepareItemCraftEvent
from .SmithItemEvent import SmithItemEvent
from .InventoryMoveItemEvent import InventoryMoveItemEvent
from .InventoryCloseEvent import InventoryCloseEvent
from .PrepareInventoryResultEvent import PrepareInventoryResultEvent
from .InventoryPickupItemEvent import InventoryPickupItemEvent
from .InventoryOpenEvent import InventoryOpenEvent


"""
Python module generated from Java source file org.yaml.snakeyaml.nodes.Tag

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.math import BigDecimal
from java.math import BigInteger
from java.util import Date
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.nodes import *
from org.yaml.snakeyaml.util import UriEncoder
from typing import Any, Callable, Iterable, Tuple


class Tag:

    PREFIX = "tag:yaml.org,2002:"
    YAML = Tag(PREFIX + "yaml")
    MERGE = Tag(PREFIX + "merge")
    SET = Tag(PREFIX + "set")
    PAIRS = Tag(PREFIX + "pairs")
    OMAP = Tag(PREFIX + "omap")
    BINARY = Tag(PREFIX + "binary")
    INT = Tag(PREFIX + "int")
    FLOAT = Tag(PREFIX + "float")
    TIMESTAMP = Tag(PREFIX + "timestamp")
    BOOL = Tag(PREFIX + "bool")
    NULL = Tag(PREFIX + "null")
    STR = Tag(PREFIX + "str")
    SEQ = Tag(PREFIX + "seq")
    MAP = Tag(PREFIX + "map")
    standardTags = HashSet<>(15)
    COMMENT = Tag(PREFIX + "comment")


    def __init__(self, tag: str):
        ...


    def __init__(self, clazz: type["Object"]):
        ...


    def isSecondary(self) -> bool:
        ...


    def getValue(self) -> str:
        ...


    def startsWith(self, prefix: str) -> bool:
        ...


    def getClassName(self) -> str:
        ...


    def toString(self) -> str:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def isCompatible(self, clazz: type[Any]) -> bool:
        """
        Java has more than 1 class compatible with a language-independent tag (!!int, !!float,
        !!timestamp etc)

        Arguments
        - clazz: - Class to check compatibility

        Returns
        - True when the Class can be represented by this language-independent tag
        """
        ...


    def matches(self, clazz: type["Object"]) -> bool:
        """
        Check whether this tag matches the global tag for the Class

        Arguments
        - clazz: - Class to check

        Returns
        - True when this tag can be used as a global tag for the Class during serialisation
        """
        ...


    def isCustomGlobal(self) -> bool:
        """
        Check if the that is global and not standard to provide it to TagInspector for verification.

        Returns
        - True when the tag must be verified to avoid remote code invocation
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.nodes.AnchorNode

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.nodes import *
from typing import Any, Callable, Iterable, Tuple


class AnchorNode(Node):
    """
    This class is only used during representation (dumping)
    """

    def __init__(self, realNode: "Node"):
        """
        Anchor

        Arguments
        - realNode: - the node which contains the referenced data
        """
        ...


    def getNodeId(self) -> "NodeId":
        ...


    def getRealNode(self) -> "Node":
        """
        Getter

        Returns
        - node with data
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.UnmodifiableSortedMultiset

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.common.collect.Multisets import UnmodifiableMultiset
from com.google.errorprone.annotations.concurrent import LazyInit
from java.util import Comparator
from java.util import NavigableSet
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class UnmodifiableSortedMultiset(UnmodifiableMultiset, SortedMultiset):
    """
    Implementation of Multisets.unmodifiableSortedMultiset(SortedMultiset), split out into
    its own file so it can be GWT emulated (to deal with the differing elementSet() types in GWT and
    non-GWT).

    Author(s)
    - Louis Wasserman
    """

    def comparator(self) -> "Comparator"["E"]:
        ...


    def elementSet(self) -> "NavigableSet"["E"]:
        ...


    def descendingMultiset(self) -> "SortedMultiset"["E"]:
        ...


    def firstEntry(self) -> "Entry"["E"]:
        ...


    def lastEntry(self) -> "Entry"["E"]:
        ...


    def pollFirstEntry(self) -> "Entry"["E"]:
        ...


    def pollLastEntry(self) -> "Entry"["E"]:
        ...


    def headMultiset(self, upperBound: "E", boundType: "BoundType") -> "SortedMultiset"["E"]:
        ...


    def subMultiset(self, lowerBound: "E", lowerBoundType: "BoundType", upperBound: "E", upperBoundType: "BoundType") -> "SortedMultiset"["E"]:
        ...


    def tailMultiset(self, lowerBound: "E", boundType: "BoundType") -> "SortedMultiset"["E"]:
        ...



"""
Python module generated from Java source file com.google.gson.internal.reflect.ReflectionHelper

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import JsonIOException
from com.google.gson.internal import GsonBuildConfig
from com.google.gson.internal import TroubleshootingGuide
from com.google.gson.internal.reflect import *
from java.lang.reflect import AccessibleObject
from java.lang.reflect import Constructor
from java.lang.reflect import Field
from java.lang.reflect import Method
from java.lang.reflect import Modifier
from typing import Any, Callable, Iterable, Tuple


class ReflectionHelper:

    @staticmethod
    def makeAccessible(object: "AccessibleObject") -> None:
        """
        Internal implementation of making an AccessibleObject accessible.

        Arguments
        - object: the object that AccessibleObject.setAccessible(boolean) should be called
            on.

        Raises
        - JsonIOException: if making the object accessible fails
        """
        ...


    @staticmethod
    def getAccessibleObjectDescription(object: "AccessibleObject", uppercaseFirstLetter: bool) -> str:
        """
        Returns a short string describing the AccessibleObject in a human-readable way. The
        result is normally shorter than AccessibleObject.toString() because it omits modifiers
        (e.g. `final`) and uses simple names for constructor and method parameter types.

        Arguments
        - object: object to describe
        - uppercaseFirstLetter: whether the first letter of the description should be uppercased
        """
        ...


    @staticmethod
    def fieldToString(field: "Field") -> str:
        """
        Creates a string representation for a field, omitting modifiers and the field type.
        """
        ...


    @staticmethod
    def constructorToString(constructor: "Constructor"[Any]) -> str:
        """
        Creates a string representation for a constructor. E.g.: `java.lang.String(char[], int,
        int)`
        """
        ...


    @staticmethod
    def isStatic(clazz: type[Any]) -> bool:
        ...


    @staticmethod
    def isAnonymousOrNonStaticLocal(clazz: type[Any]) -> bool:
        """
        Returns whether the class is anonymous or a non-static local class.
        """
        ...


    @staticmethod
    def tryMakeAccessible(constructor: "Constructor"[Any]) -> str:
        """
        Tries making the constructor accessible, returning an exception message if this fails.

        Arguments
        - constructor: constructor to make accessible

        Returns
        - exception message; `null` if successful, non-`null` if unsuccessful
        """
        ...


    @staticmethod
    def isRecord(raw: type[Any]) -> bool:
        """
        If records are supported on the JVM, this is equivalent to a call to Class.isRecord()
        """
        ...


    @staticmethod
    def getRecordComponentNames(raw: type[Any]) -> list[str]:
        ...


    @staticmethod
    def getAccessor(raw: type[Any], field: "Field") -> "Method":
        """
        Looks up the record accessor method that corresponds to the given record field
        """
        ...


    @staticmethod
    def getCanonicalRecordConstructor(raw: type["T"]) -> "Constructor"["T"]:
        ...


    @staticmethod
    def createExceptionForUnexpectedIllegalAccess(exception: "IllegalAccessException") -> "RuntimeException":
        ...



"""
Python module generated from Java source file com.google.common.cache.LongAddable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.cache import *
from typing import Any, Callable, Iterable, Tuple


class LongAddable:
    """
    Abstract interface for objects that can concurrently add longs.

    Author(s)
    - Louis Wasserman
    """

    def increment(self) -> None:
        ...


    def add(self, x: int) -> None:
        ...


    def sum(self) -> int:
        ...



"""
Python module generated from Java source file com.google.common.collect.JdkBackedImmutableSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class JdkBackedImmutableSet(IndexedImmutableSet):
    """
    ImmutableSet implementation backed by a JDK HashSet, used to defend against apparent hash
    flooding. This implementation is never used on the GWT client side, but it must be present there
    for serialization to work.

    Author(s)
    - Louis Wasserman
    """

    def contains(self, object: "Object") -> bool:
        ...


    def size(self) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.NamespacedKey

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Locale
from java.util import UUID
from org.bukkit import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class NamespacedKey:
    """
    Represents a String based key which consists of two components - a namespace
    and a key.
    
    Namespaces may only contain lowercase alphanumeric characters, periods,
    underscores, and hyphens.
    
    Keys may only contain lowercase alphanumeric characters, periods,
    underscores, hyphens, and forward slashes.
    """

    MINECRAFT = "minecraft"
    """
    The namespace representing all inbuilt keys.
    """
    BUKKIT = "bukkit"
    """
    The namespace representing all keys generated by Bukkit for backwards
    compatibility measures.
    """


    def __init__(self, namespace: str, key: str):
        """
        Create a key in a specific namespace.

        Arguments
        - namespace: namespace
        - key: key

        Unknown Tags
        - should never be used by plugins, for internal use only!!
        """
        ...


    def __init__(self, plugin: "Plugin", key: str):
        """
        Create a key in the plugin's namespace.
        
        Namespaces may only contain lowercase alphanumeric characters, periods,
        underscores, and hyphens.
        
        Keys may only contain lowercase alphanumeric characters, periods,
        underscores, hyphens, and forward slashes.

        Arguments
        - plugin: the plugin to use for the namespace
        - key: the key to create
        """
        ...


    def getNamespace(self) -> str:
        ...


    def getKey(self) -> str:
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def toString(self) -> str:
        ...


    @staticmethod
    def randomKey() -> "NamespacedKey":
        """
        Return a new random key in the .BUKKIT namespace.

        Returns
        - new key

        Deprecated
        - should never be used by plugins, for internal use only!!
        """
        ...


    @staticmethod
    def minecraft(key: str) -> "NamespacedKey":
        """
        Get a key in the Minecraft namespace.

        Arguments
        - key: the key to use

        Returns
        - new key in the Minecraft namespace
        """
        ...


    @staticmethod
    def fromString(string: str, defaultNamespace: "Plugin") -> "NamespacedKey":
        """
        Get a NamespacedKey from the supplied string with a default namespace if
        a namespace is not defined. This is a utility method meant to fetch a
        NamespacedKey from user input. Please note that casing does matter and
        any instance of uppercase characters will be considered invalid. The
        input contract is as follows:
        ```
        fromString("foo", plugin) -> "plugin:foo"
        fromString("foo:bar", plugin) -> "foo:bar"
        fromString(":foo", null) -> "minecraft:foo"
        fromString("foo", null) -> "minecraft:foo"
        fromString("Foo", plugin) -> null
        fromString(":Foo", plugin) -> null
        fromString("foo:bar:bazz", plugin) -> null
        fromString("", plugin) -> null
        ```

        Arguments
        - string: the string to convert to a NamespacedKey
        - defaultNamespace: the default namespace to use if none was
        supplied. If null, the `minecraft` namespace
        (.minecraft(String)) will be used

        Returns
        - the created NamespacedKey. null if invalid key

        See
        - .fromString(String)
        """
        ...


    @staticmethod
    def fromString(key: str) -> "NamespacedKey":
        """
        Get a NamespacedKey from the supplied string.
        
        The default namespace will be Minecraft's (i.e.
        .minecraft(String)).

        Arguments
        - key: the key to convert to a NamespacedKey

        Returns
        - the created NamespacedKey. null if invalid

        See
        - .fromString(String, Plugin)
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.components.consumable.ConsumableComponent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import Sound
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.inventory.meta.components.consumable import *
from org.bukkit.inventory.meta.components.consumable.effects import ConsumableEffect
from typing import Any, Callable, Iterable, Tuple


class ConsumableComponent(ConfigurationSerializable):
    """
    Represents a component which item can be consumed on use.
    """

    def getConsumeSeconds(self) -> float:
        """
        Gets the time in seconds it will take for this item to be consumed.

        Returns
        - consume time
        """
        ...


    def setConsumeSeconds(self, consumeSeconds: float) -> None:
        """
        Sets the time in seconds it will take for this item to be consumed.

        Arguments
        - consumeSeconds: new consume time
        """
        ...


    def getAnimation(self) -> "Animation":
        """
        Gets the animation used during consumption of the item.

        Returns
        - animation
        """
        ...


    def setAnimation(self, animation: "Animation") -> None:
        """
        Sets the animation used during consumption of the item.

        Arguments
        - animation: the new animation
        """
        ...


    def getSound(self) -> "Sound":
        """
        Gets the sound to play during and on completion of the item's
        consumption.

        Returns
        - the sound
        """
        ...


    def setSound(self, sound: "Sound") -> None:
        """
        Sets the sound to play during and on completion of the item's
        consumption.

        Arguments
        - sound: sound or null for current default
        """
        ...


    def hasConsumeParticles(self) -> bool:
        """
        Gets whether consumption particles are emitted while consuming this item.

        Returns
        - True for particles emitted while consuming, False otherwise
        """
        ...


    def setConsumeParticles(self, consumeParticles: bool) -> None:
        """
        Sets whether consumption particles are emitted while consuming this item.

        Arguments
        - consumeParticles: if particles need to be emitted while consuming
        the item
        """
        ...


    def getEffects(self) -> list["ConsumableEffect"]:
        """
        Gets the effects which may be applied by this item when consumed.

        Returns
        - consumable effects
        """
        ...


    def setEffects(self, effects: list["ConsumableEffect"]) -> None:
        """
        Sets the effects which may be applied by this item when consumed.

        Arguments
        - effects: new effects
        """
        ...


    def addEffect(self, effect: "ConsumableEffect") -> "ConsumableEffect":
        """
        Adds an effect which may be applied by this item when consumed.

        Arguments
        - effect: the effect

        Returns
        - the added effect
        """
        ...


    class Animation(Enum):
        """
        Represents the animations for an item being consumed.
        """

        DRINK = 0
        EAT = 1
        NONE = 2
        BLOCK = 3
        BOW = 4
        BRUSH = 5
        CROSSBOW = 6
        SPEAR = 7
        SPYGLASS = 8
        TOOT_HORN = 9



"""
Python module generated from Java source file org.bukkit.event.entity.TrialSpawnerSpawnEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import TrialSpawner
from org.bukkit.entity import Entity
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class TrialSpawnerSpawnEvent(EntitySpawnEvent):
    """
    Called when an entity is spawned into a world by a trial spawner.
    
    If a Trial Spawner Spawn event is cancelled, the entity will not spawn.
    """

    def __init__(self, spawnee: "Entity", spawner: "TrialSpawner"):
        ...


    def getTrialSpawner(self) -> "TrialSpawner":
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Jukebox

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import BlockData
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Jukebox(BlockData):
    """
    'has_record' is a quick flag to check whether this jukebox has a record
    inside it.
    """

    def hasRecord(self) -> bool:
        """
        Gets the value of the 'has_record' property.

        Returns
        - the 'has_record' value
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.ExplosionPrimeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import Explosive
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class ExplosionPrimeEvent(EntityEvent, Cancellable):
    """
    Called when an entity has made a decision to explode.
    """

    def __init__(self, what: "Entity", radius: float, fire: bool):
        ...


    def __init__(self, explosive: "Explosive"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getRadius(self) -> float:
        """
        Gets the radius of the explosion

        Returns
        - returns the radius of the explosion
        """
        ...


    def setRadius(self, radius: float) -> None:
        """
        Sets the radius of the explosion

        Arguments
        - radius: the radius of the explosion
        """
        ...


    def getFire(self) -> bool:
        """
        Gets whether this explosion will create fire or not

        Returns
        - True if this explosion will create fire
        """
        ...


    def setFire(self, fire: bool) -> None:
        """
        Sets whether this explosion will create fire or not

        Arguments
        - fire: True if you want this explosion to create fire
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.SortedMultiset

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.util import Comparator
from java.util import Iterator
from java.util import NavigableSet
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class SortedMultiset(SortedMultisetBridge, SortedIterable):
    """
    A Multiset which maintains the ordering of its elements, according to either their
    natural order or an explicit Comparator. This order is reflected when iterating over the
    sorted multiset, either directly, or through its `elementSet` or `entrySet` views. In
    all cases, this implementation uses Comparable.compareTo or Comparator.compare
    instead of Object.equals to determine equivalence of instances.
    
    **Warning:** The comparison must be *consistent with equals* as explained by the
    Comparable class specification. Otherwise, the resulting multiset will violate the Collection contract, which is specified in terms of Object.equals.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#multiset">`Multiset`</a>.

    Author(s)
    - Louis Wasserman

    Since
    - 11.0
    """

    def comparator(self) -> "Comparator"["E"]:
        """
        Returns the comparator that orders this multiset, or Ordering.natural() if the natural
        ordering of the elements is used.
        """
        ...


    def firstEntry(self) -> "Entry"["E"]:
        """
        Returns the entry of the first element in this multiset, or `null` if this multiset is
        empty.
        """
        ...


    def lastEntry(self) -> "Entry"["E"]:
        """
        Returns the entry of the last element in this multiset, or `null` if this multiset is
        empty.
        """
        ...


    def pollFirstEntry(self) -> "Entry"["E"]:
        """
        Returns and removes the entry associated with the lowest element in this multiset, or returns
        `null` if this multiset is empty.
        """
        ...


    def pollLastEntry(self) -> "Entry"["E"]:
        """
        Returns and removes the entry associated with the greatest element in this multiset, or returns
        `null` if this multiset is empty.
        """
        ...


    def elementSet(self) -> "NavigableSet"["E"]:
        """
        Returns a NavigableSet view of the distinct elements in this multiset.

        Since
        - 14.0 (present with return type `SortedSet` since 11.0)
        """
        ...


    def entrySet(self) -> set["Entry"["E"]]:
        """
        
        
        The `entrySet`'s iterator returns entries in ascending element order according to this
        multiset's comparator.
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        
        
        The iterator returns the elements in ascending order according to this multiset's
        comparator.
        """
        ...


    def descendingMultiset(self) -> "SortedMultiset"["E"]:
        """
        Returns a descending view of this multiset. Modifications made to either map will be reflected
        in the other.
        """
        ...


    def headMultiset(self, upperBound: "E", boundType: "BoundType") -> "SortedMultiset"["E"]:
        """
        Returns a view of this multiset restricted to the elements less than `upperBound`,
        optionally including `upperBound` itself. The returned multiset is a view of this
        multiset, so changes to one will be reflected in the other. The returned multiset supports all
        operations that this multiset supports.
        
        The returned multiset will throw an IllegalArgumentException on attempts to add
        elements outside its range.
        """
        ...


    def subMultiset(self, lowerBound: "E", lowerBoundType: "BoundType", upperBound: "E", upperBoundType: "BoundType") -> "SortedMultiset"["E"]:
        """
        Returns a view of this multiset restricted to the range between `lowerBound` and `upperBound`. The returned multiset is a view of this multiset, so changes to one will be
        reflected in the other. The returned multiset supports all operations that this multiset
        supports.
        
        The returned multiset will throw an IllegalArgumentException on attempts to add
        elements outside its range.
        
        This method is equivalent to `tailMultiset(lowerBound,
        lowerBoundType).headMultiset(upperBound, upperBoundType)`.
        """
        ...


    def tailMultiset(self, lowerBound: "E", boundType: "BoundType") -> "SortedMultiset"["E"]:
        """
        Returns a view of this multiset restricted to the elements greater than `lowerBound`,
        optionally including `lowerBound` itself. The returned multiset is a view of this
        multiset, so changes to one will be reflected in the other. The returned multiset supports all
        operations that this multiset supports.
        
        The returned multiset will throw an IllegalArgumentException on attempts to add
        elements outside its range.
        """
        ...



"""
Python module generated from Java source file com.google.gson.stream.JsonToken

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.stream import *
from enum import Enum
from typing import Any, Callable, Iterable, Tuple


class JsonToken(Enum):
    """
    A structure, name or value type in a JSON-encoded string.

    Author(s)
    - Jesse Wilson

    Since
    - 1.6
    """

    BEGIN_ARRAY = 0
    """
    The opening of a JSON array. Written using JsonWriter.beginArray and read using JsonReader.beginArray.
    """
    END_ARRAY = 1
    """
    The closing of a JSON array. Written using JsonWriter.endArray and read using JsonReader.endArray.
    """
    BEGIN_OBJECT = 2
    """
    The opening of a JSON object. Written using JsonWriter.beginObject and read using
    JsonReader.beginObject.
    """
    END_OBJECT = 3
    """
    The closing of a JSON object. Written using JsonWriter.endObject and read using JsonReader.endObject.
    """
    NAME = 4
    """
    A JSON property name. Within objects, tokens alternate between names and their values. Written
    using JsonWriter.name and read using JsonReader.nextName
    """
    STRING = 5
    """
    A JSON string.
    """
    NUMBER = 6
    """
    A JSON number represented in this API by a Java `double`, `long`, or `int`.
    """
    BOOLEAN = 7
    """
    A JSON `True` or `False`.
    """
    NULL = 8
    """
    A JSON `null`.
    """
    END_DOCUMENT = 9
    """
    The end of the JSON stream. This sentinel value is returned by JsonReader.peek() to
    signal that the JSON-encoded value has no more tokens.
    """



"""
Python module generated from Java source file org.bukkit.block.data.type.CaveVines

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import Ageable
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class CaveVines(Ageable, CaveVinesPlant):





"""
Python module generated from Java source file com.google.gson.annotations.Since

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import GsonBuilder
from com.google.gson.annotations import *
from typing import Any, Callable, Iterable, Tuple


class Since:
    """
    An annotation that indicates the version number since a member or a type has been present. This
    annotation is useful to manage versioning of your JSON classes for a web-service.
    
    This annotation has no effect unless you build com.google.gson.Gson with a `GsonBuilder` and invoke the GsonBuilder.setVersion(double) method.
    
    Here is an example of how this annotation is meant to be used:
    
    ```
    public class User {
      private String firstName;
      private String lastName;
      &#64;Since(1.0) private String emailAddress;
      &#64;Since(1.0) private String password;
      &#64;Since(1.1) private Address address;
    }
    ```
    
    If you created Gson with `new Gson()`, the `toJson()` and `fromJson()`
    methods will use all the fields for serialization and deserialization. However, if you created
    Gson with `Gson gson = new GsonBuilder().setVersion(1.0).create()` then the `toJson()` and `fromJson()` methods of Gson will exclude the `address` field since
    it's version number is set to `1.1`.

    Author(s)
    - Joel Leitch

    See
    - Until
    """

    def value(self) -> float:
        """
        The value indicating a version number since this member or type has been present. The number is
        inclusive; annotated elements will be included if `gsonVersion >= value`.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.SculkCatalyst

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.event.entity import EntityDeathEvent
from typing import Any, Callable, Iterable, Tuple


class SculkCatalyst(TileState):
    """
    Represents a captured state of a sculk catalyst.
    """

    def bloom(self, block: "Block", charges: int) -> None:
        """
        Causes a new sculk bloom, as if an entity just died around this catalyst.
        
        Typically, charges should be set to the exp reward of a mob
        (EntityDeathEvent.getDroppedExp()), which is usually
        3-5 for animals, and 5-10 for the average mob (up to 50 for
        wither skeletons). Roughly speaking, for each charge, 1 more
        sculk block will be placed.
        
        If `charges > 1000`, multiple cursors will be spawned in the
        block.

        Arguments
        - block: which block to spawn the cursor in
        - charges: how much charge to spawn.
        """
        ...



"""
Python module generated from Java source file java.security.PrivilegedAction

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.security import *
from typing import Any, Callable, Iterable, Tuple


class PrivilegedAction:
    """
    A computation to be performed with privileges enabled.  The computation is
    performed by invoking `AccessController.doPrivileged` on the
    `PrivilegedAction` object.  This interface is used only for
    computations that do not throw checked exceptions; computations that
    throw checked exceptions must use `PrivilegedExceptionAction`
    instead.

    See
    - PrivilegedExceptionAction

    Since
    - 1.2
    """

    def run(self) -> "T":
        """
        Performs the computation.  This method will be called by
        `AccessController.doPrivileged` after enabling privileges.

        Returns
        - a class-dependent value that may represent the results of the
                computation. Each class that implements
                `PrivilegedAction`
                should document what (if anything) this value represents.

        See
        - AccessController.doPrivileged(PrivilegedAction,
                                            AccessControlContext)
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.TimeLimiter

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotMock
from java.time import Duration
from java.util.concurrent import Callable
from java.util.concurrent import ExecutionException
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class TimeLimiter:
    """
    Imposes a time limit on method calls.

    Author(s)
    - Jens Nyman

    Since
    - 1.0
    """

    def newProxy(self, target: "T", interfaceType: type["T"], timeoutDuration: int, timeoutUnit: "TimeUnit") -> "T":
        """
        Returns an instance of `interfaceType` that delegates all method calls to the `target` object, enforcing the specified time limit on each call. This time-limited delegation
        is also performed for calls to Object.equals, Object.hashCode, and Object.toString.
        
        If the target method call finishes before the limit is reached, the return value or
        exception is propagated to the caller exactly as-is. If, on the other hand, the time limit is
        reached, the proxy will attempt to abort the call to the target, and will throw an UncheckedTimeoutException to the caller.
        
        It is important to note that the primary purpose of the proxy object is to return control to
        the caller when the timeout elapses; aborting the target method call is of secondary concern.
        The particular nature and strength of the guarantees made by the proxy is
        implementation-dependent. However, it is important that each of the methods on the target
        object behaves appropriately when its thread is interrupted.
        
        For example, to return the value of `target.someMethod()`, but substitute `DEFAULT_VALUE` if this method call takes over 50 ms, you can use this code:
        
        ```
          TimeLimiter limiter = . . .;
          TargetType proxy = limiter.newProxy(
              target, TargetType.class, 50, TimeUnit.MILLISECONDS);
          try {
            return proxy.someMethod();
          } catch (UncheckedTimeoutException e) {
            return DEFAULT_VALUE;
          }
        ```

        Arguments
        - target: the object to proxy
        - interfaceType: the interface you wish the returned proxy to implement
        - timeoutDuration: with timeoutUnit, the maximum length of time that callers are willing to
            wait on each method call to the proxy
        - timeoutUnit: with timeoutDuration, the maximum length of time that callers are willing to
            wait on each method call to the proxy

        Returns
        - a time-limiting proxy

        Raises
        - IllegalArgumentException: if `interfaceType` is a regular class, enum, or
            annotation type, rather than an interface
        """
        ...


    def newProxy(self, target: "T", interfaceType: type["T"], timeout: "Duration") -> "T":
        """
        Returns an instance of `interfaceType` that delegates all method calls to the `target` object, enforcing the specified time limit on each call. This time-limited delegation
        is also performed for calls to Object.equals, Object.hashCode, and Object.toString.
        
        If the target method call finishes before the limit is reached, the return value or
        exception is propagated to the caller exactly as-is. If, on the other hand, the time limit is
        reached, the proxy will attempt to abort the call to the target, and will throw an UncheckedTimeoutException to the caller.
        
        It is important to note that the primary purpose of the proxy object is to return control to
        the caller when the timeout elapses; aborting the target method call is of secondary concern.
        The particular nature and strength of the guarantees made by the proxy is
        implementation-dependent. However, it is important that each of the methods on the target
        object behaves appropriately when its thread is interrupted.
        
        For example, to return the value of `target.someMethod()`, but substitute `DEFAULT_VALUE` if this method call takes over 50 ms, you can use this code:
        
        ```
          TimeLimiter limiter = . . .;
          TargetType proxy = limiter.newProxy(target, TargetType.class, Duration.ofMillis(50));
          try {
            return proxy.someMethod();
          } catch (UncheckedTimeoutException e) {
            return DEFAULT_VALUE;
          }
        ```

        Arguments
        - target: the object to proxy
        - interfaceType: the interface you wish the returned proxy to implement
        - timeout: the maximum length of time that callers are willing to wait on each method call
            to the proxy

        Returns
        - a time-limiting proxy

        Raises
        - IllegalArgumentException: if `interfaceType` is a regular class, enum, or
            annotation type, rather than an interface

        Since
        - 28.0
        """
        ...


    def callWithTimeout(self, callable: "Callable"["T"], timeoutDuration: int, timeoutUnit: "TimeUnit") -> "T":
        """
        Invokes a specified Callable, timing out after the specified time limit. If the target method
        call finishes before the limit is reached, the return value or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the call to
        the target, and throw a TimeoutException to the caller.

        Arguments
        - callable: the Callable to execute
        - timeoutDuration: with timeoutUnit, the maximum length of time to wait
        - timeoutUnit: with timeoutDuration, the maximum length of time to wait

        Returns
        - the result returned by the Callable

        Raises
        - TimeoutException: if the time limit is reached
        - InterruptedException: if the current thread was interrupted during execution
        - ExecutionException: if `callable` throws a checked exception
        - UncheckedExecutionException: if `callable` throws a `RuntimeException`
        - ExecutionError: if `callable` throws an `Error`

        Since
        - 22.0
        """
        ...


    def callWithTimeout(self, callable: "Callable"["T"], timeout: "Duration") -> "T":
        """
        Invokes a specified Callable, timing out after the specified time limit. If the target method
        call finishes before the limit is reached, the return value or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the call to
        the target, and throw a TimeoutException to the caller.

        Arguments
        - callable: the Callable to execute
        - timeout: the maximum length of time to wait

        Returns
        - the result returned by the Callable

        Raises
        - TimeoutException: if the time limit is reached
        - InterruptedException: if the current thread was interrupted during execution
        - ExecutionException: if `callable` throws a checked exception
        - UncheckedExecutionException: if `callable` throws a `RuntimeException`
        - ExecutionError: if `callable` throws an `Error`

        Since
        - 28.0
        """
        ...


    def callUninterruptiblyWithTimeout(self, callable: "Callable"["T"], timeoutDuration: int, timeoutUnit: "TimeUnit") -> "T":
        """
        Invokes a specified Callable, timing out after the specified time limit. If the target method
        call finishes before the limit is reached, the return value or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the call to
        the target, and throw a TimeoutException to the caller.
        
        The difference with .callWithTimeout(Callable, long, TimeUnit) is that this method
        will ignore interrupts on the current thread.

        Arguments
        - callable: the Callable to execute
        - timeoutDuration: with timeoutUnit, the maximum length of time to wait
        - timeoutUnit: with timeoutDuration, the maximum length of time to wait

        Returns
        - the result returned by the Callable

        Raises
        - TimeoutException: if the time limit is reached
        - ExecutionException: if `callable` throws a checked exception
        - UncheckedExecutionException: if `callable` throws a `RuntimeException`
        - ExecutionError: if `callable` throws an `Error`

        Since
        - 22.0
        """
        ...


    def callUninterruptiblyWithTimeout(self, callable: "Callable"["T"], timeout: "Duration") -> "T":
        """
        Invokes a specified Callable, timing out after the specified time limit. If the target method
        call finishes before the limit is reached, the return value or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the call to
        the target, and throw a TimeoutException to the caller.
        
        The difference with .callWithTimeout(Callable, Duration) is that this method will
        ignore interrupts on the current thread.

        Arguments
        - callable: the Callable to execute
        - timeout: the maximum length of time to wait

        Returns
        - the result returned by the Callable

        Raises
        - TimeoutException: if the time limit is reached
        - ExecutionException: if `callable` throws a checked exception
        - UncheckedExecutionException: if `callable` throws a `RuntimeException`
        - ExecutionError: if `callable` throws an `Error`

        Since
        - 28.0
        """
        ...


    def runWithTimeout(self, runnable: "Runnable", timeoutDuration: int, timeoutUnit: "TimeUnit") -> None:
        """
        Invokes a specified Runnable, timing out after the specified time limit. If the target method
        run finishes before the limit is reached, this method returns or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the run, and
        throw a TimeoutException to the caller.

        Arguments
        - runnable: the Runnable to execute
        - timeoutDuration: with timeoutUnit, the maximum length of time to wait
        - timeoutUnit: with timeoutDuration, the maximum length of time to wait

        Raises
        - TimeoutException: if the time limit is reached
        - InterruptedException: if the current thread was interrupted during execution
        - UncheckedExecutionException: if `runnable` throws a `RuntimeException`
        - ExecutionError: if `runnable` throws an `Error`

        Since
        - 22.0
        """
        ...


    def runWithTimeout(self, runnable: "Runnable", timeout: "Duration") -> None:
        """
        Invokes a specified Runnable, timing out after the specified time limit. If the target method
        run finishes before the limit is reached, this method returns or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the run, and
        throw a TimeoutException to the caller.

        Arguments
        - runnable: the Runnable to execute
        - timeout: the maximum length of time to wait

        Raises
        - TimeoutException: if the time limit is reached
        - InterruptedException: if the current thread was interrupted during execution
        - UncheckedExecutionException: if `runnable` throws a `RuntimeException`
        - ExecutionError: if `runnable` throws an `Error`

        Since
        - 28.0
        """
        ...


    def runUninterruptiblyWithTimeout(self, runnable: "Runnable", timeoutDuration: int, timeoutUnit: "TimeUnit") -> None:
        """
        Invokes a specified Runnable, timing out after the specified time limit. If the target method
        run finishes before the limit is reached, this method returns or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the run, and
        throw a TimeoutException to the caller.
        
        The difference with .runWithTimeout(Runnable, long, TimeUnit) is that this method
        will ignore interrupts on the current thread.

        Arguments
        - runnable: the Runnable to execute
        - timeoutDuration: with timeoutUnit, the maximum length of time to wait
        - timeoutUnit: with timeoutDuration, the maximum length of time to wait

        Raises
        - TimeoutException: if the time limit is reached
        - UncheckedExecutionException: if `runnable` throws a `RuntimeException`
        - ExecutionError: if `runnable` throws an `Error`

        Since
        - 22.0
        """
        ...


    def runUninterruptiblyWithTimeout(self, runnable: "Runnable", timeout: "Duration") -> None:
        """
        Invokes a specified Runnable, timing out after the specified time limit. If the target method
        run finishes before the limit is reached, this method returns or a wrapped exception is
        propagated. If, on the other hand, the time limit is reached, we attempt to abort the run, and
        throw a TimeoutException to the caller.
        
        The difference with .runWithTimeout(Runnable, Duration) is that this method will
        ignore interrupts on the current thread.

        Arguments
        - runnable: the Runnable to execute
        - timeout: the maximum length of time to wait

        Raises
        - TimeoutException: if the time limit is reached
        - UncheckedExecutionException: if `runnable` throws a `RuntimeException`
        - ExecutionError: if `runnable` throws an `Error`

        Since
        - 28.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Hopper

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import BlockFace
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Hopper(MaterialData, Directional, Redstone):
    """
    Represents a hopper in an active or deactivated state and facing in a
    specific direction.

    See
    - Material.HOPPER

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        """
        Constructs a hopper facing the default direction (down) and initially
        active.
        """
        ...


    def __init__(self, facingDirection: "BlockFace"):
        """
        Constructs a hopper facing the specified direction and initially active.

        Arguments
        - facingDirection: the direction the hopper is facing

        See
        - BlockFace
        """
        ...


    def __init__(self, facingDirection: "BlockFace", isActive: bool):
        """
        Constructs a hopper facing the specified direction and either active or
        not.

        Arguments
        - facingDirection: the direction the hopper is facing
        - isActive: True if the hopper is initially active, False if
        deactivated

        See
        - BlockFace
        """
        ...


    def __init__(self, type: "Material"):
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def setActive(self, isActive: bool) -> None:
        """
        Sets whether the hopper is active or not.

        Arguments
        - isActive: True if the hopper is active, False if deactivated as if
        powered by redstone
        """
        ...


    def isActive(self) -> bool:
        """
        Checks whether the hopper is active or not.

        Returns
        - True if the hopper is active, False if deactivated
        """
        ...


    def setFacingDirection(self, face: "BlockFace") -> None:
        """
        Sets the direction this hopper is facing

        Arguments
        - face: The direction to set this hopper to

        See
        - BlockFace
        """
        ...


    def getFacing(self) -> "BlockFace":
        """
        Gets the direction this hopper is facing

        Returns
        - The direction this hopper is facing

        See
        - BlockFace
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "Hopper":
        ...


    def isPowered(self) -> bool:
        """
        Checks if the hopper is powered.

        Returns
        - True if the hopper is powered
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.PeekingIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotMock
from java.util import Iterator
from java.util import NoSuchElementException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class PeekingIterator(Iterator):
    """
    An iterator that supports a one-element lookahead while iterating.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CollectionHelpersExplained#peekingiterator">`PeekingIterator`</a>.

    Author(s)
    - Mick Killianey

    Since
    - 2.0
    """

    def peek(self) -> "E":
        """
        Returns the next element in the iteration, without advancing the iteration.
        
        Calls to `peek()` should not change the state of the iteration, except that it
        *may* prevent removal of the most recent element via .remove().

        Raises
        - NoSuchElementException: if the iteration has no more elements according to .hasNext()
        """
        ...


    def next(self) -> "E":
        """
        
        
        The objects returned by consecutive calls to .peek() then .next() are
        guaranteed to be equal to each other.
        """
        ...


    def remove(self) -> None:
        """
        
        
        Implementations may or may not support removal when a call to .peek() has occurred
        since the most recent call to .next().

        Raises
        - IllegalStateException: if there has been a call to .peek() since the most recent
            call to .next() and this implementation does not support this sequence of calls
            (optional)
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.TextDisplay

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import Color
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class TextDisplay(Display):
    """
    Represents a text display entity.
    """

    def getText(self) -> str:
        """
        Gets the displayed text.

        Returns
        - the displayed text.
        """
        ...


    def setText(self, text: str) -> None:
        """
        Sets the displayed text.

        Arguments
        - text: the new text
        """
        ...


    def getLineWidth(self) -> int:
        """
        Gets the maximum line width before wrapping.

        Returns
        - the line width
        """
        ...


    def setLineWidth(self, width: int) -> None:
        """
        Sets the maximum line width before wrapping.

        Arguments
        - width: new line width
        """
        ...


    def getBackgroundColor(self) -> "Color":
        """
        Gets the text background color.

        Returns
        - the background color
        """
        ...


    def setBackgroundColor(self, color: "Color") -> None:
        """
        Sets the text background color.

        Arguments
        - color: new background color
        """
        ...


    def getTextOpacity(self) -> int:
        """
        Gets the text opacity.

        Returns
        - opacity or -1 if not set
        """
        ...


    def setTextOpacity(self, opacity: int) -> None:
        """
        Sets the text opacity.

        Arguments
        - opacity: new opacity or -1 if default
        """
        ...


    def isShadowed(self) -> bool:
        """
        Gets if the text is shadowed.

        Returns
        - shadow status
        """
        ...


    def setShadowed(self, shadow: bool) -> None:
        """
        Sets if the text is shadowed.

        Arguments
        - shadow: if shadowed
        """
        ...


    def isSeeThrough(self) -> bool:
        """
        Gets if the text is see through.

        Returns
        - see through status
        """
        ...


    def setSeeThrough(self, seeThrough: bool) -> None:
        """
        Sets if the text is see through.

        Arguments
        - seeThrough: if see through
        """
        ...


    def isDefaultBackground(self) -> bool:
        """
        Gets if the text has its default background.

        Returns
        - default background
        """
        ...


    def setDefaultBackground(self, defaultBackground: bool) -> None:
        """
        Sets if the text has its default background.

        Arguments
        - defaultBackground: if default
        """
        ...


    def getAlignment(self) -> "TextAlignment":
        """
        Gets the text alignment for this display.

        Returns
        - text alignment
        """
        ...


    def setAlignment(self, alignment: "TextAlignment") -> None:
        """
        Sets the text alignment for this display.

        Arguments
        - alignment: new alignment
        """
        ...


    class TextAlignment(Enum):
        """
        Represents possible text alignments for this display.
        """

        CENTER = 0
        """
        Center aligned text (default).
        """
        LEFT = 1
        """
        Left aligned text.
        """
        RIGHT = 2
        """
        Right aligned text.
        """



"""
Python module generated from Java source file java.text.ParsePosition

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.text import *
from typing import Any, Callable, Iterable, Tuple


class ParsePosition:

    def __init__(self, index: int):
        """
        Create a new ParsePosition with the given initial index.

        Arguments
        - index: initial index
        """
        ...


    def getIndex(self) -> int:
        """
        Retrieve the current parse position.  On input to a parse method, this
        is the index of the character at which parsing will begin; on output, it
        is the index of the character following the last character parsed.

        Returns
        - the current parse position
        """
        ...


    def setIndex(self, index: int) -> None:
        """
        Set the current parse position.

        Arguments
        - index: the current parse position
        """
        ...


    def setErrorIndex(self, ei: int) -> None:
        """
        Set the index at which a parse error occurred.  Formatters
        should set this before returning an error code from their
        parseObject method.  The default value is -1 if this is not set.

        Arguments
        - ei: the index at which an error occurred

        Since
        - 1.2
        """
        ...


    def getErrorIndex(self) -> int:
        """
        Retrieve the index at which an error occurred, or -1 if the
        error index has not been set.

        Returns
        - the index at which an error occurred

        Since
        - 1.2
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Overrides equals
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hash code for this ParsePosition.

        Returns
        - a hash code value for this object
        """
        ...


    def toString(self) -> str:
        """
        Return a string representation of this ParsePosition.

        Returns
        - a string representation of this object
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.BlockData

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Color
from org.bukkit import Location
from org.bukkit import Material
from org.bukkit import Server
from org.bukkit import SoundGroup
from org.bukkit.block import Block
from org.bukkit.block import BlockFace
from org.bukkit.block import BlockState
from org.bukkit.block import BlockSupport
from org.bukkit.block import PistonMoveReaction
from org.bukkit.block.data import *
from org.bukkit.block.structure import Mirror
from org.bukkit.block.structure import StructureRotation
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BlockData(Cloneable):

    def getMaterial(self) -> "Material":
        """
        Get the Material represented by this block data.

        Returns
        - the material
        """
        ...


    def getAsString(self) -> str:
        """
        Gets a string, which when passed into a method such as
        Server.createBlockData(java.lang.String) will unambiguously
        recreate this instance.

        Returns
        - serialized data string for this block
        """
        ...


    def getAsString(self, hideUnspecified: bool) -> str:
        """
        Gets a string, which when passed into a method such as
        Server.createBlockData(java.lang.String) will recreate this or a
        similar instance where unspecified states (if any) may be optionally
        omitted. If this instance was parsed and states are omitted, this exact
        instance will be creatable when parsed again, else their equality cannot
        be guaranteed.
        
        This method will only take effect for BlockData instances created by
        methods such as Server.createBlockData(String) or any similar
        method whereby states are optionally defined. If otherwise, the result of
        .getAsString() will be returned. The following behaviour would be
        expected:
        ````String dataString = "minecraft:chest[waterlogged=True]"
        BlockData data = Bukkit.createBlockData(dataString);
        dataString.equals(data.getAsString(True)); // This would return True
        dataString.equals(data.getAsString(False)); // This would return False as all states are present
        dataString.equals(data.getAsString()); // This is equivalent to the above, "getAsString(False)"````

        Arguments
        - hideUnspecified: True if unspecified states should be omitted,
        False if they are to be shown as performed by .getAsString().

        Returns
        - serialized data string for this block
        """
        ...


    def merge(self, data: "BlockData") -> "BlockData":
        """
        Merges all explicitly set states from the given data with this BlockData.
        
        Note that the given data MUST have been created from one of the String
        parse methods, e.g. Server.createBlockData(java.lang.String) and
        not have been subsequently modified.
        
        Note also that the block types must match identically.

        Arguments
        - data: the data to merge from

        Returns
        - a new instance of this blockdata with the merged data
        """
        ...


    def matches(self, data: "BlockData") -> bool:
        """
        Checks if the specified BlockData matches this block data.
        
        The semantics of this method are such that for manually created or
        modified BlockData it has the same effect as
        Object.equals(java.lang.Object), whilst for parsed data (that to
        which .merge(org.bukkit.block.data.BlockData) applies, it will
        return True when the type and all explicitly set states match.
        
        **Note that these semantics mean that a.matches(b) may not be the same
        as b.matches(a)**

        Arguments
        - data: the data to match against (normally a parsed constant)

        Returns
        - if there is a match
        """
        ...


    def clone(self) -> "BlockData":
        """
        Returns a copy of this BlockData.

        Returns
        - a copy of the block data
        """
        ...


    def getSoundGroup(self) -> "SoundGroup":
        """
        Gets the block's SoundGroup which can be used to get its step
        sound, hit sound, and others.

        Returns
        - the sound effect group
        """
        ...


    def getLightEmission(self) -> int:
        """
        Get the amount of light emitted by this state when in the world.

        Returns
        - the light emission
        """
        ...


    def isOccluding(self) -> bool:
        """
        Check whether or not this state will occlude other blocks.
        
        Block state occlusion affects visual features of other blocks (e.g. leaves and
        wet sponges will not spawn dripping water particles if an occluding state is
        below it), or whether light will pass through it.

        Returns
        - True if occluding, False otherwise
        """
        ...


    def requiresCorrectToolForDrops(self) -> bool:
        """
        Check whether or not this state requires a specific item to be used to drop
        items when broken. For example, diamond ore requires an iron pickaxe and will
        not drop diamonds when broken with a wooden or stone pickaxe.

        Returns
        - True if a more specific item is required for drops, False if any item
        (or an empty hand) will drop items
        """
        ...


    def isPreferredTool(self, tool: "ItemStack") -> bool:
        """
        Returns if the given item is a preferred choice to break this Block.
        
        In some cases this determines if a block will drop anything or extra
        loot.

        Arguments
        - tool: The tool or item used for breaking this block

        Returns
        - True if the tool is preferred for breaking this block.
        """
        ...


    def getPistonMoveReaction(self) -> "PistonMoveReaction":
        """
        Returns the reaction of the block when moved by a piston

        Returns
        - reaction
        """
        ...


    def isSupported(self, block: "Block") -> bool:
        """
        Checks if this state would be properly supported if it were placed at
        the given Block.
        
        This may be useful, for instance, to check whether or not a wall torch is
        capable of surviving on its neighbouring block states.

        Arguments
        - block: the block position at which the state would be placed

        Returns
        - True if the block is supported, False if this state would not survive
        the world conditions
        """
        ...


    def isSupported(self, location: "Location") -> bool:
        """
        Checks if this state would be properly supported if it were placed at
        the block at the given Location.
        
        This may be useful, for instance, to check whether or not a wall torch is
        capable of surviving on its neighbouring block states.

        Arguments
        - location: the location at which the state would be placed

        Returns
        - True if the block is supported, False if this state would not survive
        the world conditions
        """
        ...


    def isFaceSturdy(self, face: "BlockFace", support: "BlockSupport") -> bool:
        """
        Checks if a state's BlockFace is capable of providing a given level
        of BlockSupport for neighbouring block states.
        
        Any given state may support either none, one, or more than one level of block
        support depending on its states. A common example would be a wall's ability to support
        torches only on the center of the upper block face, whereas a grass block would
        support all levels of block support on all block faces.

        Arguments
        - face: the face to check
        - support: the possible support level

        Returns
        - True if the face is sturdy and can support a block, False otherwise
        """
        ...


    def getMapColor(self) -> "Color":
        """
        Gets the color this block should appear as when rendered on a map.

        Returns
        - the color associated with this BlockData
        """
        ...


    def getPlacementMaterial(self) -> "Material":
        """
        Gets the material that a player would use to place this block.
        
        For most blocks this is the same as .getMaterial() but some blocks
        have different materials used to place them.
        
        For example:
        ```
        Material.REDSTONE_WIRE -> Material.REDSTONE
        Material.CARROTS -> Material.CARROT
        ```

        Returns
        - placement material
        """
        ...


    def rotate(self, rotation: "StructureRotation") -> None:
        """
        Rotates this blockdata by the specified StructureRotation.
        
        This has no effect on blocks that do not have any rotatable states.

        Arguments
        - rotation: the rotation
        """
        ...


    def mirror(self, mirror: "Mirror") -> None:
        """
        Mirrors this blockdata using the specified Mirror.
        
        This has no effect on blocks that do not have any mirrorable states.

        Arguments
        - mirror: the mirror
        """
        ...


    def copyTo(self, other: "BlockData") -> None:
        """
        Copies all applicable properties from this BlockData to the provided
        BlockData.
        
        Only modifies properties that both blocks share in common.

        Arguments
        - other: the BlockData to copy properties to
        """
        ...


    def createBlockState(self) -> "BlockState":
        """
        Creates a new default BlockState for this type of Block, not
        bound to a location.

        Returns
        - a new BlockState
        """
        ...



"""
Python module generated from Java source file org.bukkit.map.MapRenderer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.map import *
from typing import Any, Callable, Iterable, Tuple


class MapRenderer:
    """
    Represents a renderer for a map.
    """

    def __init__(self):
        """
        Initialize the map renderer base to be non-contextual. See .isContextual().
        """
        ...


    def __init__(self, contextual: bool):
        """
        Initialize the map renderer base with the given contextual status.

        Arguments
        - contextual: Whether the renderer is contextual. See .isContextual().
        """
        ...


    def isContextual(self) -> bool:
        """
        Get whether the renderer is contextual, i.e. has different canvases for
        different players.

        Returns
        - True if contextual, False otherwise.
        """
        ...


    def initialize(self, map: "MapView") -> None:
        """
        Initialize this MapRenderer for the given map.

        Arguments
        - map: The MapView being initialized.
        """
        ...


    def render(self, map: "MapView", canvas: "MapCanvas", player: "Player") -> None:
        """
        Render to the given map.

        Arguments
        - map: The MapView being rendered to.
        - canvas: The canvas to use for rendering.
        - player: The player who triggered the rendering.
        """
        ...



from .Mob import Mob
from .Zoglin import Zoglin
from .Parrot import Parrot
from .Spellcaster import Spellcaster
from .Squid import Squid
from .Warden import Warden
from .Breedable import Breedable
from .Minecart import Minecart
from .Boss import Boss
from .Turtle import Turtle
from .Spider import Spider
from .Stray import Stray
from .Husk import Husk
from .Ghast import Ghast
from .GlowSquid import GlowSquid
from .AbstractHorse import AbstractHorse
from .Hanging import Hanging
from .Trident import Trident
from .Explosive import Explosive
from .EnderDragonPart import EnderDragonPart
from .Enderman import Enderman
from .EnderPearl import EnderPearl
from .Armadillo import Armadillo
from .Axolotl import Axolotl
from .Creeper import Creeper
from .AbstractVillager import AbstractVillager
from .ShulkerBullet import ShulkerBullet
from .Shearable import Shearable
from .Chicken import Chicken
from .PufferFish import PufferFish
from .Marker import Marker
from .Ageable import Ageable
from .Evoker import Evoker
from .Wither import Wither
from .Bogged import Bogged
from .EnderSignal import EnderSignal
from .Goat import Goat
from .Slime import Slime
from .ZombieVillager import ZombieVillager
from .Ambient import Ambient
from .Vehicle import Vehicle
from .Steerable import Steerable
from .Hoglin import Hoglin
from .Item import Item
from .Snowball import Snowball
from .Giant import Giant
from .LingeringPotion import LingeringPotion
from .Wolf import Wolf
from .SmallFireball import SmallFireball
from .DragonFireball import DragonFireball
from .MagmaCube import MagmaCube
from .EntitySnapshot import EntitySnapshot
from .Strider import Strider
from .Villager import Villager
from .EntityCategory import EntityCategory
from .Pillager import Pillager
from .Vex import Vex
from .AbstractSkeleton import AbstractSkeleton
from .LightningStrike import LightningStrike
from .Player import Player
from .Pig import Pig
from .Salmon import Salmon
from .Flying import Flying
from .Endermite import Endermite
from .Sniffer import Sniffer
from .Mule import Mule
from .Projectile import Projectile
from .WanderingTrader import WanderingTrader
from .FishHook import FishHook
from .SpectralArrow import SpectralArrow
from .LeashHitch import LeashHitch
from .TraderLlama import TraderLlama
from .Panda import Panda
from .Rabbit import Rabbit
from .Frog import Frog
from .WitherSkull import WitherSkull
from .ThrownExpBottle import ThrownExpBottle
from .Bee import Bee
from .Sittable import Sittable
from .TippedArrow import TippedArrow
from .Display import Display
from .Vindicator import Vindicator
from .Boat import Boat
from .SplashPotion import SplashPotion
from .Phantom import Phantom
from .Arrow import Arrow
from .GlowItemFrame import GlowItemFrame
from .Tameable import Tameable
from .ComplexLivingEntity import ComplexLivingEntity
from .Entity import Entity
from .Camel import Camel
from .EnderCrystal import EnderCrystal
from .HumanEntity import HumanEntity
from .SpawnCategory import SpawnCategory
from .ElderGuardian import ElderGuardian
from .Dolphin import Dolphin
from .Fireball import Fireball
from .Llama import Llama
from .OminousItemSpawner import OminousItemSpawner
from .Skeleton import Skeleton
from .Tadpole import Tadpole
from .Blaze import Blaze
from .Donkey import Donkey
from .WindCharge import WindCharge
from .LivingEntity import LivingEntity
from .Egg import Egg
from .PolarBear import PolarBear
from .ChestedHorse import ChestedHorse
from .Horse import Horse
from .IronGolem import IronGolem
from .Allay import Allay
from .Cod import Cod
from .ComplexEntityPart import ComplexEntityPart
from .Illager import Illager
from .Raider import Raider
from .Ravager import Ravager
from .CaveSpider import CaveSpider
from .Drowned import Drowned
from .Illusioner import Illusioner
from .ExperienceOrb import ExperienceOrb
from .Interaction import Interaction
from .Damageable import Damageable
from .AbstractWindCharge import AbstractWindCharge
from .Cow import Cow
from .MushroomCow import MushroomCow
from .Animals import Animals
from .Bat import Bat
from .LlamaSpit import LlamaSpit
from .Silverfish import Silverfish
from .Painting import Painting
from .WitherSkeleton import WitherSkeleton
from .Creaking import Creaking
from .Ocelot import Ocelot
from .EnderDragon import EnderDragon
from .ThrownPotion import ThrownPotion
from .Monster import Monster
from .TextDisplay import TextDisplay
from .SkeletonHorse import SkeletonHorse
from .BreezeWindCharge import BreezeWindCharge
from .EntityType import EntityType
from .Cat import Cat
from .FallingBlock import FallingBlock
from .Fox import Fox
from .PiglinBrute import PiglinBrute
from .AnimalTamer import AnimalTamer
from .WaterMob import WaterMob
from .TropicalFish import TropicalFish
from .PigZombie import PigZombie
from .Piglin import Piglin
from .ItemDisplay import ItemDisplay
from .Snowman import Snowman
from .Zombie import Zombie
from .Enemy import Enemy
from .EntityFactory import EntityFactory
from .Guardian import Guardian
from .ChestBoat import ChestBoat
from .ThrowableProjectile import ThrowableProjectile
from .PiglinAbstract import PiglinAbstract
from .Witch import Witch
from .Fish import Fish
from .ItemFrame import ItemFrame
from .AreaEffectCloud import AreaEffectCloud
from .TNTPrimed import TNTPrimed
from .LargeFireball import LargeFireball
from .ArmorStand import ArmorStand
from .EvokerFangs import EvokerFangs
from .Breeze import Breeze
from .Creature import Creature
from .Firework import Firework
from .Golem import Golem
from .AbstractArrow import AbstractArrow
from .Sheep import Sheep
from .ZombieHorse import ZombieHorse
from .Shulker import Shulker
from .Pose import Pose
from .BlockDisplay import BlockDisplay
from .NPC import NPC
from .SizedFireball import SizedFireball